window.addEventListener("load", function(e) { timeshift.onLoad(e); }, false);
window.addEventListener("unload", function(e) { timeshift.prefObserver.unregister(); }, false);

var timeshift = {
	timeshiftPrefs: Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.timeshift."),
	stringsBundle: '',
	sites: [],
	prefSites: '',
	optDateFormat: '',
	optTimeFormat: '',
	optShowOld: false,
	optSet: false,

	prefObserver:
	{
		register: function()
		{
			var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
			this._branch = prefService.getBranch("extensions.timeshift.");
			this._branch.QueryInterface(Components.interfaces.nsIPrefBranch2);
			this._branch.addObserver("", this, false);
  		},

		unregister: function()
		{
			if(!this._branch) return;
			this._branch.removeObserver("", this);
		},

		observe: function(aSubject, aTopic, aData)
		{
			if(aTopic == "nsPref:changed")
			{
				timeshift.prefReader();
			}
		}
	},

	site: function(number, site, offset, format, order)
	{
		this.site = site;
		this.offset = offset;
		this.format = format;
		this.order = order;
		return this;
	},

	prefReader: function()
	{
		// Create sites list array
		var count = 0;
		timeshift.sites = new Array();
		timeshift.prefSites = '';
		try {timeshift.prefSites = timeshift.timeshiftPrefs.getCharPref("sites");}catch (err) {err = "No user sites:"+err;}
		if(timeshift.prefSites != "")
		{
			var posHeader = timeshift.prefSites.lastIndexOf('.HEADER.');
			var optHeaderVer = 1;
			if(posHeader > 0)
			{
				var optHeader = timeshift.prefSites.split('.HEADER.')[0];  // Get the header
				optHeaderVer = optHeader.valueOf();
				timeshift.prefSites = timeshift.prefSites.substring(posHeader+8);  // Strip the header from the pref
			}
			var optSites = timeshift.prefSites.split('.NEXT.');
			for (var i=0; i < optSites.length; i++)
			{
				var optSitesPieces = optSites[i].split('.ITEM.');
				// Updates any existing sites pref to the latest version - is not saved (added in 0.0.3)
				if(optHeaderVer == 2)
				{
					timeshift.sites[count] = new timeshift.site(count++, optSitesPieces[0], optSitesPieces[1], optSitesPieces[2], optSitesPieces[3]);
				} else if(optHeaderVer == 1)
				{
					timeshift.sites[count] = new timeshift.site(count++, optSitesPieces[0], optSitesPieces[1], optSitesPieces[2], '0');
				}
			}
		}
		// Read date and time format options
		timeshift.optDateFormat = '';
		timeshift.optTimeFormat = '';
		timeshift.optShowOld = false;
		try {timeshift.optDateFormat = timeshift.timeshiftPrefs.getCharPref("DateFormat");}catch (err) {err = "No user date format:"+err;}
		try {timeshift.optTimeFormat = timeshift.timeshiftPrefs.getCharPref("TimeFormat");}catch (err) {err = "No user time format:"+err;}
		try {timeshift.optShowOld = timeshift.timeshiftPrefs.getBoolPref("ShowOld");}catch (err) {err = "No user show old:"+err;}
		if (timeshift.prefSites != '' && timeshift.optDateFormat != '' && timeshift.optTimeFormat != '')
		{
			timeshift.optSet = true;
		} else
		{
			timeshift.optSet = false;
		}
	},

	onLoad: function()
	{
		var appcontent = document.getElementById("appcontent");   // browser
		if (appcontent)
		{
			appcontent.addEventListener("DOMContentLoaded", timeshift.onPageLoad, false);
		}

		// Register the preference observer
		timeshift.prefObserver.register();

		// Read string bundle set
		timeshift.stringsBundle = document.getElementById('string-bundle');

		// load the preferences
		timeshift.prefReader();
	},

	onPageLoad: function(aEvent) 
	{
		var doc = aEvent.originalTarget; // doc is document that triggered "onload" event

		/* If options are set then continue */
		if (timeshift.optSet == true)
		{
			var thisSite = doc.location.href;
			for(var i=0; i < timeshift.sites.length; i++)
			{
				if (thisSite.search(timeshift.sites[i].site) > -1)
				{
					timeshift.findText(doc, timeshift.sites[i]);  //  Start the search
					break;
				}
			}
		}
	},

	// Loop through the document text nodes for evaluation
	findText: function(doc, optSite)
	{
		var node = doc.body; // Get the body node (we don't want to stuff around with headers)
		var textNodes = doc.evaluate("//text()", node, null, 6, null);
		var textNode;

		for (var i = 0; textNode = textNodes.snapshotItem(i); i++)
		{
			if (textNode.parentNode.nodeName != 'SCRIPT')  //  Do not stuff with anything inside a script
			{
				timeshift.replaceDates(doc, textNode, optSite);
			}
		} 
	},

	// Searches text nodes for dates and or times to shift
	replaceDates: function(doc, node, optSite)
	{
		if (optSite.order == 0) // Order is Date/Time
		{
			var node_date = node.nodeValue.match(/(?:(?:(?:SUN|MON|TUE(?:S)?|WED(?:NES)?|THU(?:RS)?|FRI|SAT(?:UR)?)(?:DAY)?(?:&NBSP;|\s)*)?(?:\d{1,2}(?:\/\d{1,2}\/|(?:ST|ND|RD|TH|&NBSP;|\s)*(?:JAN(?:UARY)*|feb(?:RUARY)*|MAR(?:CH)*|APR(?:IL)*|MAY|JUN(?:E)*|JUL(?:Y)*|AUG(?:UST)*|SEP(?:T(?:EMBER)*)*|OCT(?:OBER)*|NOV(?:EMBER)*|DEC(?:EMBER)*)(?:&NBSP;|\s)*)\d{2,2}(?:\d{2,2})?)|\d{2,2}(?:\d{2,2})?-\d{2,2}-\d{2,2})(?:&NBSP;|\s)*(?:(?:\d{1,2}:){1,2}\d{1,2}(?:(?:&NBSP;|\s)*[AP]+M?)?)?|(?:(?:\d{1,2}:){1,2}\d{1,2}(?:(?:&NBSP;|\s)*[AP]+M?)?)/gi);  // Get the dates and or times from the node
		} else // Order is Time/Date
		{
			var node_date = node.nodeValue.match(/(?:(?:\d{1,2}:){1,2}\d{1,2}(?:(?:&NBSP;|\s)*[AP]+M?)?)?(?:&NBSP;|\s)*(?:(?:(?:SUN|MON|TUE(?:S)?|WED(?:NES)?|THU(?:RS)?|FRI|SAT(?:UR)?)(?:DAY)?(?:&NBSP;|\s)*)?(?:\d{1,2}(?:\/\d{1,2}\/|(?:ST|ND|RD|TH|&NBSP;|\s)*(?:JAN(?:UARY)*|feb(?:RUARY)*|MAR(?:CH)*|APR(?:IL)*|MAY|JUN(?:E)*|JUL(?:Y)*|AUG(?:UST)*|SEP(?:T(?:EMBER)*)*|OCT(?:OBER)*|NOV(?:EMBER)*|DEC(?:EMBER)*)(?:&NBSP;|\s)*)\d{2,2}(?:\d{2,2})?)|\d{2,2}(?:\d{2,2})?-\d{2,2}-\d{2,2})|(?:(?:\d{1,2}:){1,2}\d{1,2}(?:(?:&NBSP;|\s)*[AP]+M?)?)/gi);  // Get the dates and or times from the node
		}
		if (node_date != null)
		{
			for (var i = 0; i < node_date.length; i++)  // Loop through the dates and/or times
			{
				node_date[i] = node_date[i].replace(/^(?:\s|&nbsp;)+/g, '').replace(/(?:\s|&nbsp;)+$/g, '');  //  Strip any leading or trailing white space characters the regexp may have picked up
				var stringDate = '1/1/1'; // Set the base date, only needed because you cannot create a date object with just a time in Javascript
				var shiftDate = false; // Only becomes true if there is a date to shift
				var node_date_part = node_date[i].match(/(\d{1,2})\/(\d{1,2})\/(\d{2,2}(?:\d{2,2})?)/);  //  Look for dd/mm/yy(yy) or mm/dd/yy(yy) dates
				if (node_date_part != null)
				{
					if (optSite.format == 2)  // If site uses US format
					{
						stringDate = node_date_part[1] + '\/' + node_date_part[2] + '\/' + timeshift.fixShortYear(node_date_part[3]);
					} else  // It uses non US
					{
						stringDate = node_date_part[2] + '\/' + node_date_part[1] + '\/' + timeshift.fixShortYear(node_date_part[3]);
					}
					shiftDate = true;
				} else
				{
					var node_date_part = node_date[i].match(/(\d{1,2})(?:ST|ND|RD|TH|&NBSP;|\s)*((?:JAN(?:UARY)*|feb(?:RUARY)*|MAR(?:CH)*|APR(?:IL)*|MAY|JUN(?:E)*|JUL(?:Y)*|AUG(?:UST)*|SEP(?:T(?:EMBER)*)*|OCT(?:OBER)*|NOV(?:EMBER)*|DEC(?:EMBER)*))(?:&NBSP;|\s)*(\d{2,2}(?:\d{2,2})?)/i);  //  Look for dd mmm(m) yy(yy) dates
					if (node_date_part != null)
					{
						stringDate = node_date_part[1] + ' ' + node_date_part[2] + ' ' + timeshift.fixShortYear(node_date_part[3]);
						shiftDate = true;
					} else
					{
						var node_date_part = node_date[i].match(/(\d{2,2}(?:\d{2,2})?)-(\d{2,2})-(\d{2,2})/);  //  Look for (yy)yy-mm-dd dates
						if (node_date_part != null)
						{
							stringDate = node_date_part[2] + '\/' + node_date_part[3] + '\/' + timeshift.fixShortYear(node_date_part[1]);
							shiftDate = true;
						}
					}
				}
				var shiftTime = false;  // Only becomes true if there is a time to shift
				var node_date_time_part = node_date[i].match(/((?:\d{1,2}:){1,2}\d{1,2})(?:(?:&NBSP;|\s)*([AP]+M?))?/);  // Get the useful bits of the string
				if (node_date_time_part != null)
				{
					node_date_time_part.shift();  // Get rid of the whole time string from the array
					stringDate += ' ' + node_date_time_part.join(' ');  // Join the time back together with only a space between the time and the am/pm
					var node_date_time_precision = stringDate.split(':');  // Split the time so we can figure the precision
					var timePrecision = node_date_time_precision.length;  // The precision eg hh:mm or hh:mm:ss etc
					shiftTime = true;
				}
				var tempDate = new Date(stringDate);  // Create a javascript date
				if (shiftTime)  // If there is a time component
				{
					tempDate = new Date(Number(tempDate) + optSite.offset*60*60*1000);  // Apply the offset
				}
				var newDate = '';  // Create the string which will replace the original date and/or time
				if (shiftDate)  // If there is a date component
				{
					newDate += timeshift.formatDate(tempDate);  // Format it
					if (shiftTime)
					{
						newDate += ' '; // Add a normal space between date and time
					}
				}
				if (shiftTime)  // If there is a time component
				{
					newDate += timeshift.formatTime(tempDate, timePrecision);  // Format it
				}
				if (timeshift.optShowOld)  // If we are showing old time on mouseover
				{
					var nodeBefore = node.nodeValue.substr(0, node.nodeValue.search(node_date[i]));  // Get the text before the date/time
					var nodeBeforeText = doc.createTextNode(nodeBefore);  // Create a new node with the text before date/time
					node.parentNode.insertBefore(nodeBeforeText, node);  // Add the new node before the current node
					var newNode = doc.createElement('SPAN');  // Create a new span node
					newNode.setAttribute('title', node_date[i]);  // Add the title attribute (original date/time)
					var newNodeText = doc.createTextNode(newDate);  // Create a new node with the new date/time
					newNode.appendChild(newNodeText);  // Add the new node to the span node
					node.parentNode.insertBefore(newNode, node);  // Add the span node before the current node
					node.nodeValue = node.nodeValue.replace(nodeBefore + node_date[i], '');  // Delete the data in the new nodes from the current node
				} else
				{
					node.nodeValue = node.nodeValue.replace(node_date[i], newDate);  // Replace the original date and/or time with the new one
				}
			}
		}
	},

	// Formats a date to the required format
	formatDate: function(sourceDate)
	{
		var sourceDay = sourceDate.getDate();
		var sourceMonth = sourceDate.getMonth() + 1;
		var sourceYear = sourceDate.getFullYear();
		if (timeshift.optDateFormat == 1)
		{
			dateReturn = timeshift.leadingZero(sourceDay) + '\/' + timeshift.leadingZero(sourceMonth) + '\/' + sourceYear;
		} else if (timeshift.optDateFormat == 2)
		{
			var dateReturn = sourceDay + '\u00a0' + timeshift.MonthReturn(sourceMonth, 'abbr') + '\u00a0' + sourceYear;
		} else if (timeshift.optDateFormat == 3)
		{
			var dateReturn = sourceDay + '\u00a0' + timeshift.MonthReturn(sourceMonth, 'full') + '\u00a0' + sourceYear;
		} else if (timeshift.optDateFormat == 4)
		{
			var dateReturn = sourceYear + '-' + timeshift.leadingZero(sourceMonth) + '-' + timeshift.leadingZero(sourceDay);
		} else if (timeshift.optDateFormat == 5)
		{
			dateReturn = timeshift.leadingZero(sourceMonth) + '\/' + timeshift.leadingZero(sourceDay) + '\/' + sourceYear;
		}
		return dateReturn;
	},

	// Formats a time to the required format
	formatTime: function(sourceTime, timePrecision)
	{
		var timeHours = sourceTime.getHours();
		var timeEnd = '';
		if (timeshift.optTimeFormat == 2)
		{
			timeEnd += '\u00a0';
			if (timeHours < 12)
			{
				timeEnd += timeshift.TimeAMPM('am');
			} else
			{
				timeHours = timeHours - 12;
				timeEnd += timeshift.TimeAMPM('pm');
			}
			if (timeHours == 0)
			{
				timeHours = 12;
			}
		}
		var timeReturn = timeshift.leadingZero(timeHours) + ':' + timeshift.leadingZero(sourceTime.getMinutes());
		if (timePrecision >= 3)
		{
			timeReturn += ':' + timeshift.leadingZero(sourceTime.getSeconds());
		}
		timeReturn += timeEnd;
		return timeReturn;
	},

	// If the year is expressed as 06 instead of 2006 then fix that
	fixShortYear: function(sourceYear)
	{
		sourceReturn = String(sourceYear);
		if (sourceReturn.length <= 2) // if it is a short year
		{
			if (Number(sourceYear) < 80)  //  Use 1980 as the cut off for 19's
			{
				sourceReturn = '20' + sourceReturn;
			} else
			{
				sourceReturn = '19' + sourceReturn;
			}
		}
		return sourceReturn;
	},

	// Add a leading zero to single digit numbers eg 5 becomes 05
	leadingZero: function(sourceNumber)
	{
		var tempNumber = '00' + String(sourceNumber);
		var returnNumber = tempNumber.substr(tempNumber.length - 2);
		return returnNumber;
	},

	// Get local month names
	MonthReturn: function(sourceMonth, sourceType)
	{
		return timeshift.stringsBundle.getString('month_' + String(sourceMonth) + '_' + sourceType);
	},

	// Get local am/pm names
	TimeAMPM: function(sourceType)
	{
		return timeshift.stringsBundle.getString('time_' + sourceType);
	},

}